@@ -37,6 +37,7 @@ static SPStyleProp const props[] = {
     {SP_ATTR_SODIPODI_INSENSITIVE, "sodipodi:insensitive"},
     {SP_ATTR_SODIPODI_NONPRINTABLE, "sodipodi:nonprintable"},
     {SP_ATTR_CONNECTOR_AVOID, "inkscape:connector-avoid"},
+    {SP_ATTR_CONNECTION_POINTS, "inkscape:connection-points"},
     {SP_ATTR_STYLE, "style"},
     {SP_ATTR_TRANSFORM_CENTER_X, "inkscape:transform-center-x"},
     {SP_ATTR_TRANSFORM_CENTER_Y, "inkscape:transform-center-y"},
@@ -57,8 +58,8 @@ static SPStyleProp const props[] = {
     {SP_ATTR_HEIGHT, "height"},
     {SP_ATTR_VIEWBOX, "viewBox"},
     {SP_ATTR_PRESERVEASPECTRATIO, "preserveAspectRatio"},
-    {SP_ATTR_SODIPODI_VERSION, "sodipodi:version"},
     {SP_ATTR_INKSCAPE_VERSION, "inkscape:version"},
+    {SP_ATTR_ONLOAD, "onload"},
     /* SPNamedView */
     {SP_ATTR_VIEWONLY, "viewonly"},
     {SP_ATTR_SHOWGUIDES, "showguides"},
@@ -76,6 +77,10 @@ static SPStyleProp const props[] = {
     {SP_ATTR_BORDERCOLOR, "bordercolor"},
     {SP_ATTR_BORDEROPACITY, "borderopacity"},
     {SP_ATTR_PAGECOLOR, "pagecolor"},
+    {SP_ATTR_FIT_MARGIN_TOP, "fit-margin-top"},
+    {SP_ATTR_FIT_MARGIN_LEFT, "fit-margin-left"},
+    {SP_ATTR_FIT_MARGIN_RIGHT, "fit-margin-right"},
+    {SP_ATTR_FIT_MARGIN_BOTTOM, "fit-margin-bottom"},
     {SP_ATTR_INKSCAPE_PAGEOPACITY, "inkscape:pageopacity"},
     {SP_ATTR_INKSCAPE_PAGESHADOW, "inkscape:pageshadow"},
     {SP_ATTR_INKSCAPE_ZOOM, "inkscape:zoom"},
@@ -85,17 +90,25 @@ static SPStyleProp const props[] = {
     {SP_ATTR_INKSCAPE_WINDOW_HEIGHT, "inkscape:window-height"},
     {SP_ATTR_INKSCAPE_WINDOW_X, "inkscape:window-x"},
     {SP_ATTR_INKSCAPE_WINDOW_Y, "inkscape:window-y"},
+    {SP_ATTR_INKSCAPE_WINDOW_MAXIMIZED, "inkscape:window-maximized"},
     {SP_ATTR_INKSCAPE_SNAP_GLOBAL, "inkscape:snap-global"},
     {SP_ATTR_INKSCAPE_SNAP_BBOX, "inkscape:snap-bbox"},
     {SP_ATTR_INKSCAPE_SNAP_NODES, "inkscape:snap-nodes"},
-    {SP_ATTR_INKSCAPE_SNAP_GUIDE, "inkscape:snap-guide"},
+    {SP_ATTR_INKSCAPE_SNAP_FROM_GUIDE, "inkscape:snap-from-guide"},
     {SP_ATTR_INKSCAPE_SNAP_CENTER, "inkscape:snap-center"},
-    {SP_ATTR_INKSCAPE_SNAP_INTERS_GRIDGUIDE, "inkscape:snap-intersection-grid-guide"},
-    {SP_ATTR_INKSCAPE_SNAP_INTERS_LINESEGM, "inkscape:snap-intersection-line-segments"},
+    {SP_ATTR_INKSCAPE_SNAP_GRIDS, "inkscape:snap-grids"},
+    {SP_ATTR_INKSCAPE_SNAP_TO_GUIDES, "inkscape:snap-to-guides"},
+    {SP_ATTR_INKSCAPE_SNAP_SMOOTH_NODES, "inkscape:snap-smooth-nodes"},
+    {SP_ATTR_INKSCAPE_SNAP_LINE_MIDPOINTS, "inkscape:snap-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_OBJECT_MIDPOINTS, "inkscape:snap-object-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_BBOX_EDGE_MIDPOINTS, "inkscape:snap-bbox-edge-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_BBOX_MIDPOINTS, "inkscape:snap-bbox-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_INTERS_PATHS, "inkscape:snap-intersection-paths"},
     {SP_ATTR_INKSCAPE_OBJECT_PATHS, "inkscape:object-paths"},
     {SP_ATTR_INKSCAPE_OBJECT_NODES, "inkscape:object-nodes"},
     {SP_ATTR_INKSCAPE_BBOX_PATHS, "inkscape:bbox-paths"},
     {SP_ATTR_INKSCAPE_BBOX_NODES, "inkscape:bbox-nodes"},
+    {SP_ATTR_INKSCAPE_SNAP_PAGE, "inkscape:snap-page"},
     {SP_ATTR_INKSCAPE_CURRENT_LAYER, "inkscape:current-layer"},
     {SP_ATTR_INKSCAPE_DOCUMENT_UNITS, "inkscape:document-units"},
     {SP_ATTR_INKSCAPE_CONNECTOR_SPACING, "inkscape:connector-spacing"},
@@ -111,11 +124,14 @@ static SPStyleProp const props[] = {
     {SP_ATTR_Y, "y"},
     /* SPPath */
     {SP_ATTR_D, "d"},
-    {SP_ATTR_INKSCAPE_ORIGINAL_D, "inkscape:original-d"},    
+    {SP_ATTR_INKSCAPE_ORIGINAL_D, "inkscape:original-d"},
     /* (Note: XML representation of connectors may change in future.) */
     {SP_ATTR_CONNECTOR_TYPE, "inkscape:connector-type"},
+    {SP_ATTR_CONNECTOR_CURVATURE, "inkscape:connector-curvature"},
     {SP_ATTR_CONNECTION_START, "inkscape:connection-start"},
     {SP_ATTR_CONNECTION_END, "inkscape:connection-end"},
+    {SP_ATTR_CONNECTION_START_POINT, "inkscape:connection-start-point"},
+    {SP_ATTR_CONNECTION_END_POINT, "inkscape:connection-end-point"},
     /* SPRect */
     {SP_ATTR_RX, "rx"},
     {SP_ATTR_RY, "ry"},
@@ -240,7 +256,7 @@ static SPStyleProp const props[] = {
     /*feFlood*/
     {SP_ATTR_FLOODCOLOR, "flood-color"},
     {SP_ATTR_FLOODOPACITY, "flood-opacity"},
-	/* SPGaussianBlur */
+    /* SPGaussianBlur */
     {SP_ATTR_STDDEVIATION, "stdDeviation"},
     /*feImage*/
     /*feMerge*/
@@ -250,7 +266,7 @@ static SPStyleProp const props[] = {
     /*feOffset*/
     //{SP_ATTR_DX, "dx"},
     //{SP_ATTR_DY, "dy"},
-	/*feSpecularLighting*/
+    /*feSpecularLighting*/
     {SP_ATTR_SPECULARCONSTANT, "specularConstant"},
     {SP_ATTR_SPECULAREXPONENT, "specularExponent"},
     /*feTile*/
@@ -264,6 +280,7 @@ static SPStyleProp const props[] = {
     {SP_ATTR_GRADIENTUNITS, "gradientUnits"},
     {SP_ATTR_GRADIENTTRANSFORM, "gradientTransform"},
     {SP_ATTR_SPREADMETHOD, "spreadMethod"},
+    {SP_ATTR_OSB_SWATCH, "osb:paint"},
     /* SPRadialGradient */
     {SP_ATTR_FX, "fx"},
     {SP_ATTR_FY, "fy"},
@@ -307,6 +324,64 @@ static SPStyleProp const props[] = {
     {SP_ATTR_ADDITIVE, "additive"},
     {SP_ATTR_ACCUMULATE, "accumulate"},
 
+    /* SVGFonts */
+    /*<font>*/
+    {SP_ATTR_HORIZ_ORIGIN_X, "horiz-origin-x"},
+    {SP_ATTR_HORIZ_ORIGIN_Y, "horiz-origin-y"},
+    {SP_ATTR_HORIZ_ADV_X, "horiz-adv-x"},
+    {SP_ATTR_VERT_ORIGIN_X, "vert-origin-x"},
+    {SP_ATTR_VERT_ORIGIN_Y, "vert-origin-y"},
+    {SP_ATTR_VERT_ADV_Y, "vert-adv-y"},
+
+    /*<glyph>*/
+    {SP_ATTR_UNICODE, "unicode"},
+    {SP_ATTR_GLYPH_NAME, "glyph-name"},
+    //{SP_ATTR_ORIENTATION, "orientation"},
+    {SP_ATTR_ARABIC_FORM, "arabic-form"},
+    {SP_ATTR_LANG, "lang"},
+
+    /*<hkern> and <vkern>*/
+    {SP_ATTR_U1, "u1"},
+    {SP_ATTR_G1, "g1"},
+    {SP_ATTR_U2, "u2"},
+    {SP_ATTR_G2, "g2"},
+    {SP_ATTR_K, "k"},
+
+    /*<font-face>*/
+    //{SP_ATTR_FONT_FAMILY, "font-family"}, these are already set for CSS2 (SP_PROP_FONT_FAMILY, SP_PROP_FONT_STYLE, SP_PROP_FONT_VARIANT etc...)
+    //{SP_ATTR_FONT_STYLE, "font-style"},
+    //{SP_ATTR_FONT_VARIANT, "font-variant"},
+    //{SP_ATTR_FONT_WEIGHT, "font-weight"},
+    //{SP_ATTR_FONT_STRETCH, "font-stretch"},
+    //{SP_ATTR_FONT_SIZE, "font-size"},
+    {SP_ATTR_UNICODE_RANGE, "unicode-range"},
+    {SP_ATTR_UNITS_PER_EM, "units-per-em"},
+    {SP_ATTR_PANOSE_1, "panose-1"},
+    {SP_ATTR_STEMV, "stemv"},
+    {SP_ATTR_STEMH, "stemh"},
+    //{SP_ATTR_SLOPE, "slope"},
+    {SP_ATTR_CAP_HEIGHT, "cap-height"},
+    {SP_ATTR_X_HEIGHT, "x-height"},
+    {SP_ATTR_ACCENT_HEIGHT, "accent-height"},
+    {SP_ATTR_ASCENT, "ascent"},
+    {SP_ATTR_DESCENT, "descent"},
+    {SP_ATTR_WIDTHS, "widths"},
+    {SP_ATTR_BBOX, "bbox"},
+    {SP_ATTR_IDEOGRAPHIC, "ideographic"},
+    {SP_ATTR_ALPHABETIC, "alphabetic"},
+    {SP_ATTR_MATHEMATICAL, "mathematical"},
+    {SP_ATTR_HANGING, "hanging"},
+    {SP_ATTR_V_IDEOGRAPHIC, "v-ideographic"},
+    {SP_ATTR_V_ALPHABETIC, "v-alphabetic"},
+    {SP_ATTR_V_MATHEMATICAL, "v-mathematical"},
+    {SP_ATTR_V_HANGING, "v-hanging"},
+    {SP_ATTR_UNDERLINE_POSITION, "underline-position"},
+    {SP_ATTR_UNDERLINE_THICKNESS, "underline-thickness"},
+    {SP_ATTR_STRIKETHROUGH_POSITION, "strikethrough-position"},
+    {SP_ATTR_STRIKETHROUGH_THICKNESS, "strikethrough-thickness"},
+    {SP_ATTR_OVERLINE_POSITION, "overline-position"},
+    {SP_ATTR_OVERLINE_THICKNESS, "overline-thickness"},
+
     /* XML */
     {SP_ATTR_XML_SPACE, "xml:space"},
 
