@@ -36,6 +36,7 @@ enum SPAttributeEnum {
     SP_ATTR_SODIPODI_INSENSITIVE,
     SP_ATTR_SODIPODI_NONPRINTABLE,
     SP_ATTR_CONNECTOR_AVOID,
+    SP_ATTR_CONNECTION_POINTS,
     SP_ATTR_STYLE,
     SP_ATTR_TRANSFORM_CENTER_X,
     SP_ATTR_TRANSFORM_CENTER_Y,
@@ -57,8 +58,8 @@ enum SPAttributeEnum {
     SP_ATTR_HEIGHT,
     SP_ATTR_VIEWBOX,
     SP_ATTR_PRESERVEASPECTRATIO,
-    SP_ATTR_SODIPODI_VERSION,
     SP_ATTR_INKSCAPE_VERSION,
+    SP_ATTR_ONLOAD,
     /* SPNamedView */
     SP_ATTR_VIEWONLY,
     SP_ATTR_SHOWGUIDES,
@@ -76,6 +77,10 @@ enum SPAttributeEnum {
     SP_ATTR_BORDERCOLOR,
     SP_ATTR_BORDEROPACITY,
     SP_ATTR_PAGECOLOR,
+    SP_ATTR_FIT_MARGIN_TOP,
+    SP_ATTR_FIT_MARGIN_LEFT,
+    SP_ATTR_FIT_MARGIN_RIGHT,
+    SP_ATTR_FIT_MARGIN_BOTTOM,
     SP_ATTR_INKSCAPE_PAGEOPACITY,
     SP_ATTR_INKSCAPE_PAGESHADOW,
     SP_ATTR_INKSCAPE_ZOOM,
@@ -85,17 +90,26 @@ enum SPAttributeEnum {
     SP_ATTR_INKSCAPE_WINDOW_HEIGHT,
     SP_ATTR_INKSCAPE_WINDOW_X,
     SP_ATTR_INKSCAPE_WINDOW_Y,
+    SP_ATTR_INKSCAPE_WINDOW_MAXIMIZED,
     SP_ATTR_INKSCAPE_SNAP_GLOBAL,
     SP_ATTR_INKSCAPE_SNAP_BBOX,
     SP_ATTR_INKSCAPE_SNAP_NODES,
-    SP_ATTR_INKSCAPE_SNAP_GUIDE,
+    SP_ATTR_INKSCAPE_SNAP_FROM_GUIDE,
     SP_ATTR_INKSCAPE_SNAP_CENTER,
-    SP_ATTR_INKSCAPE_SNAP_INTERS_GRIDGUIDE,
-    SP_ATTR_INKSCAPE_SNAP_INTERS_LINESEGM,
+    SP_ATTR_INKSCAPE_SNAP_GRIDS,
+    SP_ATTR_INKSCAPE_SNAP_TO_GUIDES,
+    SP_ATTR_INKSCAPE_SNAP_SMOOTH_NODES,
+    SP_ATTR_INKSCAPE_SNAP_LINE_MIDPOINTS,
+    SP_ATTR_INKSCAPE_SNAP_OBJECT_MIDPOINTS,
+    SP_ATTR_INKSCAPE_SNAP_BBOX_EDGE_MIDPOINTS,
+    SP_ATTR_INKSCAPE_SNAP_BBOX_MIDPOINTS,
+    //SP_ATTR_INKSCAPE_SNAP_INTERS_GRIDGUIDE,
+    SP_ATTR_INKSCAPE_SNAP_INTERS_PATHS,
     SP_ATTR_INKSCAPE_OBJECT_PATHS,
     SP_ATTR_INKSCAPE_OBJECT_NODES,
     SP_ATTR_INKSCAPE_BBOX_PATHS,
-    SP_ATTR_INKSCAPE_BBOX_NODES,    
+    SP_ATTR_INKSCAPE_BBOX_NODES,
+    SP_ATTR_INKSCAPE_SNAP_PAGE,
     SP_ATTR_INKSCAPE_CURRENT_LAYER,
     SP_ATTR_INKSCAPE_DOCUMENT_UNITS,
     SP_ATTR_INKSCAPE_CONNECTOR_SPACING,
@@ -113,8 +127,11 @@ enum SPAttributeEnum {
     SP_ATTR_D,
     SP_ATTR_INKSCAPE_ORIGINAL_D,
     SP_ATTR_CONNECTOR_TYPE,
+    SP_ATTR_CONNECTOR_CURVATURE,
     SP_ATTR_CONNECTION_START,
     SP_ATTR_CONNECTION_END,
+    SP_ATTR_CONNECTION_START_POINT,
+    SP_ATTR_CONNECTION_END_POINT,
     /* SPRect */
     SP_ATTR_RX,
     SP_ATTR_RY,
@@ -239,7 +256,7 @@ enum SPAttributeEnum {
     /*feFlood*/
     SP_ATTR_FLOODCOLOR,
     SP_ATTR_FLOODOPACITY,
-	/* SPGaussianBlur */
+    /* SPGaussianBlur */
     SP_ATTR_STDDEVIATION,
     /*feImage*/
     /*feMerge*/
@@ -249,14 +266,14 @@ enum SPAttributeEnum {
     /*feOffset*/
     //SP_ATTR_DX,
     //SP_ATTR_DY,
-	/*feSpecularLighting*/
+    /*feSpecularLighting*/
     //SP_ATTR_SURFACESCALE,
     SP_ATTR_SPECULARCONSTANT,
     SP_ATTR_SPECULAREXPONENT,
     /*feTile*/
     /*feTurbulence*/
     SP_ATTR_BASEFREQUENCY,
-	SP_ATTR_NUMOCTAVES,
+    SP_ATTR_NUMOCTAVES,
     SP_ATTR_SEED,
     SP_ATTR_STITCHTILES,
     //SP_ATTR_TYPE,
@@ -264,6 +281,7 @@ enum SPAttributeEnum {
     SP_ATTR_GRADIENTUNITS,
     SP_ATTR_GRADIENTTRANSFORM,
     SP_ATTR_SPREADMETHOD,
+    SP_ATTR_OSB_SWATCH,
     /* SPRadialGradient */
     SP_ATTR_FX,
     SP_ATTR_FY,
@@ -307,6 +325,63 @@ enum SPAttributeEnum {
     SP_ATTR_ADDITIVE,
     SP_ATTR_ACCUMULATE,
 
+    /* SVGFonts */
+    /* SPFont */
+    SP_ATTR_HORIZ_ORIGIN_X,
+    SP_ATTR_HORIZ_ORIGIN_Y,
+    SP_ATTR_HORIZ_ADV_X,
+    SP_ATTR_VERT_ORIGIN_X,
+    SP_ATTR_VERT_ORIGIN_Y,
+    SP_ATTR_VERT_ADV_Y,
+
+    SP_ATTR_UNICODE,
+    SP_ATTR_GLYPH_NAME,
+    //SP_ATTR_ORIENTATION,
+    SP_ATTR_ARABIC_FORM,
+    SP_ATTR_LANG,
+
+    /*<hkern> and <vkern>*/
+    SP_ATTR_U1,
+    SP_ATTR_G1,
+    SP_ATTR_U2,
+    SP_ATTR_G2,
+    SP_ATTR_K,
+
+    /*<font-face>*/
+//    SP_ATTR_FONT_FAMILY,
+//    SP_ATTR_FONT_STYLE,
+//    SP_ATTR_FONT_VARIANT,
+//    SP_ATTR_FONT_WEIGHT,
+//    SP_ATTR_FONT_STRETCH,
+//    SP_ATTR_FONT_SIZE,
+    SP_ATTR_UNICODE_RANGE,
+    SP_ATTR_UNITS_PER_EM,
+    SP_ATTR_PANOSE_1,
+    SP_ATTR_STEMV,
+    SP_ATTR_STEMH,
+    //SP_ATTR_SLOPE,
+    SP_ATTR_CAP_HEIGHT,
+    SP_ATTR_X_HEIGHT,
+    SP_ATTR_ACCENT_HEIGHT,
+    SP_ATTR_ASCENT,
+    SP_ATTR_DESCENT,
+    SP_ATTR_WIDTHS,
+    SP_ATTR_BBOX,
+    SP_ATTR_IDEOGRAPHIC,
+    SP_ATTR_ALPHABETIC,
+    SP_ATTR_MATHEMATICAL,
+    SP_ATTR_HANGING,
+    SP_ATTR_V_IDEOGRAPHIC,
+    SP_ATTR_V_ALPHABETIC,
+    SP_ATTR_V_MATHEMATICAL,
+    SP_ATTR_V_HANGING,
+    SP_ATTR_UNDERLINE_POSITION,
+    SP_ATTR_UNDERLINE_THICKNESS,
+    SP_ATTR_STRIKETHROUGH_POSITION,
+    SP_ATTR_STRIKETHROUGH_THICKNESS,
+    SP_ATTR_OVERLINE_POSITION,
+    SP_ATTR_OVERLINE_THICKNESS,
+
     /* XML */
     SP_ATTR_XML_SPACE,
 
