@@ -8,6 +8,8 @@
 #include <glib-object.h>
 #include <glib/gtypes.h>
 #if ENABLE_LCMS
+#include <vector>
+#include <glibmm/ustring.h>
 #include <lcms.h>
 #endif // ENABLE_LCMS
 
@@ -25,7 +27,17 @@ GType colorprofile_get_type();
 
 #if ENABLE_LCMS
 
-cmsHPROFILE colorprofile_get_handle( SPDocument* document, guint* intent, gchar* const name );
+cmsHPROFILE colorprofile_get_handle( SPDocument* document, guint* intent, gchar const* name );
+cmsHTRANSFORM colorprofile_get_display_transform();
+
+Glib::ustring colorprofile_get_display_id( int screen, int monitor );
+Glib::ustring colorprofile_set_display_per( gpointer buf, guint bufLen, int screen, int monitor );
+cmsHTRANSFORM colorprofile_get_display_per( Glib::ustring const& id );
+
+std::vector<Glib::ustring> colorprofile_get_display_names();
+std::vector<Glib::ustring> colorprofile_get_softproof_names();
+
+Glib::ustring get_path_for_profile(Glib::ustring const& name);
 
 #endif
 
