@@ -7,6 +7,7 @@
 
 #include <glib/gtypes.h>
 #include <sp-object.h>
+#include <glibmm/ustring.h>
 #if ENABLE_LCMS
 #include <lcms.h>
 #endif // ENABLE_LCMS
@@ -32,13 +33,18 @@ struct ColorProfile : public SPObject {
     static GType getType();
     static void classInit( ColorProfileClass *klass );
 
+    static std::list<Glib::ustring> getProfileDirs();
 #if ENABLE_LCMS
     static cmsHPROFILE getSRGBProfile();
+    static cmsHPROFILE getNULLProfile();
 
     icColorSpaceSignature getColorSpace() const {return _profileSpace;}
     icProfileClassSignature getProfileClass() const {return _profileClass;}
     cmsHTRANSFORM getTransfToSRGB8();
     cmsHTRANSFORM getTransfFromSRGB8();
+    cmsHTRANSFORM getTransfGamutCheck();
+    bool GamutCheck(SPColor color);
+
 #endif // ENABLE_LCMS
 
     gchar* href;
@@ -56,17 +62,19 @@ private:
     static void release( SPObject *object );
     static void build( SPObject *object, SPDocument *document, Inkscape::XML::Node *repr );
     static void set( SPObject *object, unsigned key, gchar const *value );
-    static Inkscape::XML::Node *write( SPObject *object, Inkscape::XML::Node *repr, guint flags );
+    static Inkscape::XML::Node *write( SPObject *object, Inkscape::XML::Document *doc, Inkscape::XML::Node *repr, guint flags );
 #if ENABLE_LCMS
     static DWORD _getInputFormat( icColorSpaceSignature space );
     void _clearProfile();
 
     static cmsHPROFILE _sRGBProf;
+    static cmsHPROFILE _NullProf;
 
     icProfileClassSignature _profileClass;
     icColorSpaceSignature _profileSpace;
     cmsHTRANSFORM _transf;
     cmsHTRANSFORM _revTransf;
+    cmsHTRANSFORM _gamutTransf;
 #endif // ENABLE_LCMS
 };
 
