@@ -38,10 +38,7 @@
 #include "display/rendermode.h"
 #include "display/snap-indicator.h"
 
-class NRRect;
 class SPCSSAttr;
-struct _GtkWidget;
-typedef struct _GtkWidget GtkWidget;
 struct SPCanvas;
 struct SPCanvasItem;
 struct SPCanvasGroup;
@@ -143,6 +140,11 @@ struct SPDesktop : public Inkscape::UI::View::View
     sigc::signal<void, SPObject *>     _layer_changed_signal;
     sigc::signal<bool, const SPCSSAttr *>::accumulated<StopOnTrue> _set_style_signal;
     sigc::signal<int, SPStyle *, int>::accumulated<StopOnNonZero> _query_style_signal;
+    
+    /// Emitted when the zoom factor changes (not emitted when scrolling).
+    /// The parameter is the new zoom factor
+    sigc::signal<void, double> signal_zoom_changed;
+    
     sigc::connection connectDocumentReplaced (const sigc::slot<void,SPDesktop*,SPDocument*> & slot)
     {
         return _document_replaced_signal.connect (slot);
@@ -201,6 +203,9 @@ struct SPDesktop : public Inkscape::UI::View::View
     void setDisplayModeOutline() {
         _setDisplayMode(Inkscape::RENDERMODE_OUTLINE);
     }
+//    void setDisplayModePrintColorsPreview() {
+//        _setDisplayMode(Inkscape::RENDERMODE_PRINT_COLORS_PREVIEW);
+//    }
     void displayModeToggle();
     Inkscape::RenderMode _display_mode;
     Inkscape::RenderMode getMode() const { return _display_mode; }
