@@ -17,7 +17,6 @@
 #define NR_ARENA_IMAGE(o) (NR_CHECK_INSTANCE_CAST ((o), NR_TYPE_ARENA_IMAGE, NRArenaImage))
 #define NR_IS_ARENA_IMAGE(o) (NR_CHECK_INSTANCE_TYPE ((o), NR_TYPE_ARENA_IMAGE))
 
-#include <libnr/nr-matrix.h>
 #include "nr-arena-item.h"
 #include "style.h"
 
@@ -32,10 +31,10 @@ struct NRArenaImage : public NRArenaItem {
     double x, y;
     double width, height;
 
-    NR::Point c00, c01, c11, c10; // all 4 corners of the image, for outline mode rect
+    Geom::Point c00, c01, c11, c10; // all 4 corners of the image, for outline mode rect
 
     /* From GRID to PIXELS */
-    NR::Matrix grid2px;
+    Geom::Matrix grid2px;
 
     SPStyle *style;
 
