@@ -5,7 +5,7 @@
  * feMorphology filter primitive renderer
  *
  * Authors:
- *   Felipe Corrêa da Silva Sanches <felipe.sanches@gmail.com>
+ *   Felipe Corrêa da Silva Sanches <juca@members.fsf.org>
  *
  * Copyright (C) 2007 authors
  *
@@ -16,7 +16,8 @@
 #include "display/nr-filter-slot.h"
 #include "display/nr-filter-units.h"
 
-namespace NR {
+namespace Inkscape {
+namespace Filters {
 
 enum FilterMorphologyOperator {
     MORPHOLOGY_OPERATOR_ERODE,
@@ -31,19 +32,20 @@ public:
     virtual ~FilterMorphology();
 
     virtual int render(FilterSlot &slot, FilterUnits const &units);
-    virtual void area_enlarge(NRRectL &area, Matrix const &trans);
+    virtual void area_enlarge(NRRectL &area, Geom::Matrix const &trans);
     virtual FilterTraits get_input_traits();
     void set_operator(FilterMorphologyOperator &o);
-    void set_xradius(int x);
-    void set_yradius(int y);
+    void set_xradius(double x);
+    void set_yradius(double y);
 
 private:
     FilterMorphologyOperator Operator;
-    int xradius;
-    int yradius;
+    double xradius;
+    double yradius;
 };
 
-} /* namespace NR */
+} /* namespace Filters */
+} /* namespace Inkscape */
 
 #endif /* __NR_FILTER_MORPHOLOGY_H__ */
 /*
