@@ -2,7 +2,7 @@
  * feTile filter primitive renderer
  *
  * Authors:
- *   Felipe Corrêa da Silva Sanches <felipe.sanches@gmail.com>
+ *   Felipe Corrêa da Silva Sanches <juca@members.fsf.org>
  *
  * Copyright (C) 2007 authors
  *
@@ -12,7 +12,8 @@
 #include "display/nr-filter-tile.h"
 #include "display/nr-filter-units.h"
 
-namespace NR {
+namespace Inkscape {
+namespace Filters {
 
 FilterTile::FilterTile()
 {
@@ -52,7 +53,7 @@ int FilterTile::render(FilterSlot &slot, FilterUnits const &/*units*/) {
     return 0;
 }
 
-void FilterTile::area_enlarge(NRRectL &/*area*/, Matrix const &/*trans*/)
+void FilterTile::area_enlarge(NRRectL &/*area*/, Geom::Matrix const &/*trans*/)
 {
 }
 
@@ -60,7 +61,8 @@ FilterTraits FilterTile::get_input_traits() {
     return TRAIT_PARALLER;
 }
 
-} /* namespace NR */
+} /* namespace Filters */
+} /* namespace Inkscape */
 
 /*
   Local Variables:
