@@ -35,10 +35,11 @@
 
 #include <glibmm/ustring.h>
 
-#include <libnr/nr-matrix.h>
-#include <libnr/nr-rect.h>
+#include <2geom/matrix.h>
 #include <libnr/nr-rect-l.h>
 
+#include <2geom/rect.h>
+
 G_BEGIN_DECLS
 
 struct SPCanvas;
@@ -62,6 +63,7 @@ struct SPCanvasBuf{
     guint32 bg_color;
     // If empty, ignore contents of buffer and use a solid area of bg_color
     bool is_empty;
+    cairo_t *ct;
 };
 
 /**
@@ -73,18 +75,18 @@ struct SPCanvasItem : public GtkObject {
     SPCanvasItem *parent;
 
     double x1, y1, x2, y2;
-    NR::Rect bounds;
-    NR::Matrix xform;
+    Geom::Rect bounds;
+    Geom::Matrix xform;
 };
 
 /**
  * The vtable of an SPCanvasItem.
  */
 struct _SPCanvasItemClass : public GtkObjectClass {
-    void (* update) (SPCanvasItem *item, NR::Matrix const &affine, unsigned int flags);
+    void (* update) (SPCanvasItem *item, Geom::Matrix const &affine, unsigned int flags);
 
     void (* render) (SPCanvasItem *item, SPCanvasBuf *buf);
-    double (* point) (SPCanvasItem *item, NR::Point p, SPCanvasItem **actual_item);
+    double (* point) (SPCanvasItem *item, Geom::Point p, SPCanvasItem **actual_item);
 
     int (* event) (SPCanvasItem *item, GdkEvent *event);
 };
@@ -95,16 +97,17 @@ G_END_DECLS
 
 #define sp_canvas_item_set gtk_object_set
 
-void sp_canvas_item_affine_absolute(SPCanvasItem *item, NR::Matrix const &aff);
+void sp_canvas_item_affine_absolute(SPCanvasItem *item, Geom::Matrix const &aff);
 
 void sp_canvas_item_raise(SPCanvasItem *item, int positions);
 void sp_canvas_item_lower(SPCanvasItem *item, int positions);
+bool sp_canvas_item_is_visible(SPCanvasItem *item);
 void sp_canvas_item_show(SPCanvasItem *item);
 void sp_canvas_item_hide(SPCanvasItem *item);
 int sp_canvas_item_grab(SPCanvasItem *item, unsigned int event_mask, GdkCursor *cursor, guint32 etime);
 void sp_canvas_item_ungrab(SPCanvasItem *item, guint32 etime);
 
-NR::Matrix sp_canvas_item_i2w_affine(SPCanvasItem const *item);
+Geom::Matrix sp_canvas_item_i2w_affine(SPCanvasItem const *item);
 
 void sp_canvas_item_grab_focus(SPCanvasItem *item);
 
@@ -189,7 +192,7 @@ struct SPCanvas {
 
     bool is_scrolling;
 
-    NR::Rect getViewbox() const;
+    Geom::Rect getViewbox() const;
     NR::IRect getViewboxIntegers() const;
 };
 
@@ -204,13 +207,13 @@ void sp_canvas_request_redraw(SPCanvas *canvas, int x1, int y1, int x2, int y2);
 void sp_canvas_force_full_redraw_after_interruptions(SPCanvas *canvas, unsigned int count);
 void sp_canvas_end_forced_full_redraws(SPCanvas *canvas);
 
-bool sp_canvas_world_pt_inside_window(SPCanvas const *canvas, NR::Point const &world);
+bool sp_canvas_world_pt_inside_window(SPCanvas const *canvas, Geom::Point const &world);
 
 void sp_canvas_window_to_world(SPCanvas const *canvas, double winx, double winy, double *worldx, double *worldy);
 void sp_canvas_world_to_window(SPCanvas const *canvas, double worldx, double worldy, double *winx, double *winy);
 
-NR::Point sp_canvas_window_to_world(SPCanvas const *canvas, NR::Point const win);
-NR::Point sp_canvas_world_to_window(SPCanvas const *canvas, NR::Point const world);
+Geom::Point sp_canvas_window_to_world(SPCanvas const *canvas, Geom::Point const win);
+Geom::Point sp_canvas_world_to_window(SPCanvas const *canvas, Geom::Point const world);
 
 #endif // SEEN_SP_CANVAS_H
 
