@@ -142,6 +142,16 @@ static void sp_dropper_context_finish(SPEventContext *ec)
 }
 
 
+/**
+ * Returns the current dropper context icc-color.
+ */
+SPColor* sp_dropper_context_get_icc_color(SPEventContext */*ec*/)
+{
+    //TODO: implement-me!
+
+    return 0; // At least we will cause a clean crash, instead of random corruption.
+}
+
 /**
  * Returns the current dropper context color.
  */
